@@ -1,27 +1,7 @@
 #' Function to simulate a tree, conditional on observing all states.
-#' @param lambdas speciation rates, in the form of a list of matrices
-#' @param mus extinction rates, in the form of a vector
-#' @param qs The Q matrix, for example the result of function q_doubletrans, but
-#' generally in the form of a matrix.
-#' @param num_concealed_states number of concealed states
-#' @param crown_age crown age of the tree, tree will be simulated conditional
-#' on non-extinction and this crown age.
-#' @param pool_init_states pool of initial states at the crown, in case this is
-#' different from all available states, otherwise leave at NULL
-#' @param maxSpec Maximum number of species in the tree (please note that the
-#' tree is not conditioned on this number, but that this is a safeguard against
-#' generating extremely large trees).
-#' @param conditioning can be 'obs_states', 'true_states' or 'none', the tree is
-#' simulated until one is generated that contains all observed states
-#' ('obs_states'), all true states (e.g. all combinations of obs and hidden
-#' states), or is always returned ('none').
-#' @param non_extinction should the tree be conditioned on non-extinction of the
-#' crown lineages? Default is TRUE.
-#' @param verbose provide intermediate output.
-#' @param max_tries maximum number of simulations to try to obtain a tree.
-#' @param drop_extinct should extinct species be dropped from the tree? default
-#' is TRUE.
-#' @param seed pseudo-random number generator seed
+#' 
+#' @inheritParams default_params_doc
+#'
 #' @return a list with four properties: phy: reconstructed phylogeny,
 #' true_traits: the true traits in order of tip label, obs_traits: observed
 #' traits, ignoring hidden traits and lastly:
@@ -34,7 +14,7 @@
 #' Simulation is performed with a randomly
 #' sampled initial trait at the crown - if you, however - want a specific,
 #' single, trait used at the crown, you can reduce the possible traits by
-#' modifying 'pool_init_states'.
+#' modifying `pool_init_states`.
 #'
 #' By default, the algorithm keeps simulating until it generates a tree where
 #' both crown lineages survive to the present - this is to ensure that the tree
@@ -47,8 +27,11 @@ secsse_sim <- function(lambdas,
                        crown_age,
                        num_concealed_states,
                        pool_init_states = NULL,
-                       maxSpec = 1e5,
-                       conditioning = "none",
+                       max_spec = 1e5,
+                       min_spec = 2,
+                       max_species_extant = TRUE,
+                       tree_size_hist = FALSE,
+                       conditioning = "obs_states",
                        non_extinction = TRUE,
                        verbose = FALSE,
                        max_tries = 1e6,
@@ -82,64 +65,118 @@ secsse_sim <- function(lambdas,
     # now we have to match the indices
     all_states <- names(mus)
     indices <- which(all_states %in% pool_init_states)
+    if (length(indices) < 1) {
+      # most likely states without hidden labels have been provided
+      letters_conceal <- LETTERS[1:num_concealed_states]
+      all_states2 <- c()
+      for (i in 1:length(pool_init_states)) {
+        for (j in 1:length(letters_conceal)) {
+          all_states2 <- c(all_states2, paste0(pool_init_states[i], letters_conceal[j]))
+        }
+      }
+      indices <- which(all_states %in% all_states2)
+    }
     pool_init_states <- -1 + indices
   }
 
-  if (!conditioning %in% c("none", "true_states", "obs_states")) {
-    stop("unknown conditioning, please pick from
-         'none', 'obs_states', 'true_states'")
-  }
-  
   if (is.null(seed)) seed <- -1
 
+  condition_vec <- vector()
+  if (length(conditioning) > 1) {
+    condition_vec <- conditioning
+    conditioning <- "custom"
+    true_traits <- names(mus)
+    all_states <- c()
+    for (i in seq_along(true_traits)) {
+      all_states[i] <- substr(true_traits[i], 1, (nchar(true_traits[i]) - 1))
+    }
+    all_states <- unique(all_states)
+    
+    indices <- which(all_states %in% condition_vec)
+    condition_vec <- -1 + indices
+  } 
+  
   res <- secsse_sim_cpp(mus,
                         lambdas,
                         qs,
                         crown_age,
-                        maxSpec,
+                        max_spec,
+                        max_species_extant,
+                        min_spec,
                         pool_init_states,
                         conditioning,
                         num_concealed_states,
                         non_extinction,
                         verbose,
                         max_tries,
-                        seed)
+                        seed,
+                        condition_vec,
+                        tree_size_hist)
 
-  if (length(res$traits) < 1) {
+  Ltable        <- res$ltable
+  
+  out_hist <- 0
+  if (tree_size_hist == TRUE) out_hist <- res$hist_tree_size
+  
+  if (sum(Ltable[, 4] == -1) < 2) {
     warning("crown lineages died out")
     return(list(phy = "ds",
                 traits = 0,
                 extinct = res$tracker[2],
                 overshoot = res$tracker[3],
-                conditioning = res$tracker[4]))
+                conditioning = res$tracker[4],
+                small = res$tracker[6],
+                size_hist = out_hist))
   }
 
-  Ltable        <- res$ltable
+  if (sum(res$tracker) >= max_tries) {
+    warning("Couldn't simulate a tree in enough tries,
+            try increasing max_tries")
+    
+    return(list(phy = "ds",
+                traits = 0,
+                extinct = res$tracker[2],
+                overshoot = res$tracker[3],
+                conditioning = res$tracker[4],
+                small = res$tracker[6],
+                size_hist = out_hist))
+  }
 
-  speciesID     <- res$traits[seq(2, length(res$traits), by = 2)]
-  initialState  <- res$initial_state
+  
+
+  initialState  <- names(mus)[1 + res$initial_state]
   Ltable[, 1]   <- crown_age - Ltable[, 1] # simulation starts at 0,
                                            # not at crown age
   notmin1 <- which(Ltable[, 4] != -1)
   Ltable[notmin1, 4] <- crown_age - c(Ltable[notmin1, 4])
   Ltable[which(Ltable[, 4] == crown_age + 1), 4] <- -1
 
-  indices       <- seq(1, length(res$traits), by = 2)
-  speciesTraits <- 1 + res$traits[indices]
+  # indices       <- seq(1, length(res$traits), by = 2)
+  speciesTraits <- 1 + Ltable[, 5]
+  used_id <- abs(Ltable[, 3])
 
   phy <- DDD::L2phylo(Ltable, dropextinct = drop_extinct)
+  
+ 
+  if (drop_extinct) {
+    to_drop <- which(Ltable[, 4] != -1)
+    if (length(to_drop) > 0) {
+      used_id <- used_id[-to_drop]
+      speciesTraits <- speciesTraits[-to_drop]
+    }
+  }
 
-  true_traits <- sortingtraits(data.frame(cbind(paste0("t", abs(speciesID)),
+  true_traits <- sortingtraits(data.frame(cbind(paste0("t", used_id),
                                              speciesTraits),
-                                       row.names = NULL),
-                            phy)
+                                          row.names = NULL),
+                               phy)
 
   true_traits <- names(mus)[true_traits]
   obs_traits <- c()
   for (i in seq_along(true_traits)) {
-    obs_traits[i] <- stringr::str_sub(true_traits[i], 1, -2)
+    obs_traits[i] <- substr(true_traits[i], 1, (nchar(true_traits[i]) - 1))
   }
-
+  
   if (sum(Ltable[, 4] < 0)) {
       return(list(phy = phy,
                 true_traits = true_traits,
@@ -148,8 +185,8 @@ secsse_sim <- function(lambdas,
                 extinct = res$tracker[2],
                 overshoot = res$tracker[3],
                 conditioning = res$tracker[4],
-                event_counter = res$event_counter,
-                extinct_draw = res$extinct_draw))
+                small = res$tracker[6],
+                size_hist = out_hist))
   } else {
     warning("simulation did not meet minimal requirements")
     return(list(phy = "ds",
@@ -157,7 +194,7 @@ secsse_sim <- function(lambdas,
                 extinct = res$tracker[2],
                 overshoot = res$tracker[3],
                 conditioning = res$tracker[4],
-                event_counter = res$event_counter,
-                extinct_draw = res$extinct_draw))
+                small = res$tracker[6],
+                size_hist = out_hist))
   }
 }
